using AdventureClient;
using Log;
using Newtonsoft.Json.Linq;
using Protocols;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using Utility;

namespace LobbyServer
{
    class ParseFileObj{
        public int pos;
        public int byteLength;
        public string name;
        public EHttpActionType type;
    }
    /// <summary>
    /// 操作类型
    /// </summary>
    enum EHttpActionType:int{
        UploadFile = 0,//上传文件
    }
    public enum  EUpLoadJSONDataErrorCode{
        OK = 0,
        Fail = 1,
    }
    /// <summary>
    /// JSON解析结构体
    /// </summary>
    class IUpLoadJSONData{
        /**
        * 上传类型 0 上传文件
        */
        public int type;
        /**
        * 文件存储名
        */
        public string name = "";
        /**
        * 数据 根据type适配  
        * type:0 data为 base64编码字符串
        */
        // public string data;

        public EUpLoadJSONDataErrorCode code = EUpLoadJSONDataErrorCode.Fail;
    }
    /// <summary>
    /// hptt监听类
    /// -接收http消息并处理
    /// </summary>
    public class HttpUploadListener
    {
        private const string LogTitle = "HttpUploadListener";

        /// <summary>
        /// 用户token值
        /// k=用户id v=token值
        /// </summary>
        // private Dictionary<ulong, string> userTokens = new Dictionary<ulong, string>();

        //////////////////////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// 初始化http返回
        /// </summary>
        public bool Init()
        {
            var cfg = Env.otherConfig.httpPostConfig;

            if (initListener(cfg.UploadPrefixes))
                return true;

            return false;
        }

        /// <summary>
        /// 初始化监听
        /// </summary>
        /// <param name="prefixes"></param>
        /// <returns></returns>
        private bool initListener(string prefixes)
        {
            Console.WriteLine(" in init..." + prefixes);

            if (!System.Net.HttpListener.IsSupported)
            {
                LogSys.Warn("Windows XP SP2 or Server 2003 is required to use the HttpListener class.");
                return false;
            }

            System.Net.HttpListener listener = new System.Net.HttpListener();
            listener.Prefixes.Add(prefixes);
            addAddress(prefixes, Environment.UserDomainName, Environment.UserName);
            listener.Start();//如果启动失败，以管理员身份运行

            startListen(listener);

            return true;
        }

        /// <summary>
        /// 添加监听地址
        /// </summary>
        /// <param name="address"></param>
        /// <param name="domain"></param>
        /// <param name="user"></param>
        private void addAddress(string address, string domain, string user)
        {
            string argsDll = String.Format(@"http delete urlacl url={0}", address);
            string args = string.Format(@"http add urlacl url={0} user={1}\{2}", address, domain, user);
            ProcessStartInfo psi = new ProcessStartInfo("netsh", argsDll);
            psi.Verb = "runas";
            psi.CreateNoWindow = false;
            psi.WindowStyle = ProcessWindowStyle.Hidden;
            psi.UseShellExecute = false;
            Process.Start(psi).WaitForExit();//删除urlacl
            psi = new ProcessStartInfo("netsh", args);
            psi.Verb = "runas";
            psi.CreateNoWindow = false;
            psi.WindowStyle = ProcessWindowStyle.Hidden;
            psi.UseShellExecute = false;
            Process.Start(psi).WaitForExit();//添加urlacl
        }


        public static void ByteToFile(byte[] fileData,string fileName)
        {
            // string fileName = DateTime.Now.ToString("yyyyMMddhhmmss") + ".pdf";
            // string dirPath = @"C:\Users\Administrator\Desktop\CreatePDF";
            // string dirPath = @"d:\png";
            string dirPath = Env.otherConfig.configs.upload;//上传文件路径
            if (!Directory.Exists(dirPath))
                Directory.CreateDirectory(dirPath);

            string filePath = Path.Combine(dirPath, fileName);

            using (FileStream fileStream = new FileStream(filePath, FileMode.OpenOrCreate))
            {
                fileStream.Write(fileData, 0, fileData.Length);
                Log.LogSys.Debug($"save file:{filePath}");
                fileStream.Close();
            }

        }

        private string CreateFileName(string name,string uid){
            return uid + "_" +  DateTimeHelp.DateTimeToUlong(DateTime.Now) + "_" + name;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="reader"></param>
        /// <returns></returns>
        private IUpLoadJSONData ParseLoadJson(StreamReader reader){
            IUpLoadJSONData po = new IUpLoadJSONData();
            string body = reader.ReadToEnd();
            JObject obj = JObject.Parse(body);
            JToken type = null;
            JToken name = null;
            JToken data = null;
            JToken uid = null;
            obj.TryGetValue("type", out type);
            obj.TryGetValue("name",out name);
            obj.TryGetValue("data", out data);
            obj.TryGetValue("uid", out uid);
            if (type != null)
            {
                po.type = int.Parse(type.ToString());
            }
            if (name != null && uid!=null){
                

                po.name = CreateFileName(name.ToString(),uid.ToString());
                if (data != null){
                    EHttpActionType curType = (EHttpActionType)po.type;
                    if(curType == EHttpActionType.UploadFile){
                        var bytes = Convert.FromBase64String(data.ToString());
                        double a = new Random().RandomDouble(1.0);
                        //存文件
                        ByteToFile(bytes,po.name);
                        po.code = HttpListenerHandler.ReqUploadFile(uid.ToString(),po.name);
                    }
                }
            }
            return po;
        }

        private ParseFileObj ParseJsonStr(StreamReader reader){
            ParseFileObj po = new ParseFileObj();
            const int headLen = 2;
            char[] lenChars = new char[headLen];
            reader.Read(lenChars,0,headLen);
            byte[] byteData=Encoding.UTF8.GetBytes(lenChars);  
            int length = (int)BitConverter.ToUInt16(byteData,0);
            // Console.WriteLine($"length:{length}");

            char[] jsonChars = new char[length];
            reader.Read(jsonChars,0,length);
            byte[] json = Encoding.UTF8.GetBytes(jsonChars);
            string str = System.Text.Encoding.UTF8.GetString(json);
            // Console.WriteLine($"pos:{length+2}");
            po.pos = length + headLen;//当前的文件指针位置
            JObject obj = JObject.Parse(str);

            JToken len = null;
            JToken name = null;
            JToken type = null;

            obj.TryGetValue("type", out type);
            obj.TryGetValue("len", out len);
            obj.TryGetValue("name",out name);
            // obj.TryGetValue("data", out dataToken);
            if(len!=null){
                po.byteLength = int.Parse(len.ToString());
            }
            
            if(name!=null){
                po.name = name.ToString();;
                // Console.WriteLine(name);
            }
            if(type!=null){
                po.type = (EHttpActionType)int.Parse(type.ToString());

                switch(po.type){
                    case EHttpActionType.UploadFile:
                        //上传文件
                        char[] datas = new char[po.byteLength];
                        reader.Read(datas,0,po.byteLength);


                        /*
                        byte[] dataBytes =   Encoding.ASCII.GetBytes(datas);//Encoding.Default.GetBytes(datas);//Encoding.Default.GetBytes(datas);
                        // Console.WriteLine(dataBytes.Length);
                            // byte[] bb = Encoding.ASCII.GetBytes(cc);
                        // Encoding.Convert(Encoding.UTF8,Encoding.ASCII,dataBytes);
                        HttpUploadListener.ByteToFile(dataBytes,po.name);
                        */
                        for(int i = 0;i < datas.Length;i++){
                            
                            Console.WriteLine(i+":["+datas[i]+"]"+ (uint)(datas[i]));//(uint)datas[i])
                        }
                        break;
                }
            }
            return po;
        }
        /// <summary>
        /// 开启监听 '{"id":1,"data":"yourdata"}'
        /// </summary>
        /// <param name="listener"></param>
        private async void startListen(System.Net.HttpListener listener)
        {
            HttpListenerContext context = await listener.GetContextAsync();
            HttpListenerRequest request = context.Request;
            HttpListenerResponse response = context.Response;
            response.AppendHeader("Access-Control-Allow-Origin", "*");
            // response.AppendHeader("Access-Control-Allow-Credentials", "true");
            response.AppendHeader("Access-Control-Allow-Methods", "GET,POST,OPTIONS");


            // add_header Access-Control-Allow-Credentials true;
			// add_header Access-Control-Allow-Methods 'GET,POST,OPTIONS';
            string rsp = "error";
            

            try
            {

                if (request.HttpMethod == "OPTIONS")
                {
                    //                     response.writeHead(200, {
                    //   'Content-Type': 'text/plain',
                    //   'Access-Control-Allow-Origin': '*',
                    //   'Access-Control-Allow-Headers': 'Content-Type, Content-Length, Authorization, Accept, X-Requested-With , yourHeaderFeild, sessionToken',
                    //   'Access-Control-Allow-Methods': 'PUT, POST, GET, DELETE, OPTIONS'
                    // });
                    rsp = "204";
                }
                else if(request.HttpMethod == "POST")
                {
                    using (StreamReader reader = new StreamReader(request.InputStream, Encoding.UTF8))
                    {
                        // ParseFileObj po = ParseJsonStr(reader);
                        IUpLoadJSONData result = ParseLoadJson(reader);
                        rsp = "{\"code\":" + (int)result.code + "}";
                        /*
                                            string body = reader.ReadToEnd();
                                            // ByteToFile(System.Text.Encoding.UTF8.GetBytes(body),"base64.txt");
                                            var bytes = Convert.FromBase64String(body);
                                            double a = new Random().RandomDouble(1.0);
                                            ByteToFile(bytes,a.ToString()+"base64.png");
                        */
                    }
                    //LogSys.InfoModule(HttpResponseLogName, $"获取Http的post返回的数据. rsp={rsp}");
                }
            }
            catch (Exception e)
            {
                LogSys.Error($"httpListener throw exception : {e}");
            }
            finally
            {
                using (StreamWriter writer = new StreamWriter(response.OutputStream, Encoding.UTF8)){
                    string wStr = JSONUtil.JsonCompress(rsp);//AESHelp.AESEncrypt(rsp, AESHelp.KEY);
                    writer.Write(wStr);
                    writer.Close();
                    response.Close();
                }

                startListen(listener);
            }
        }

    }
}
